local lg = assert( love.graphics )
local love = assert( love )
local scene = assert( require 'client.scene' )
local strings = assert( require 'client.assets.strings' )
local button = assert( require 'client.ui.button' )
local textinput = assert( require 'client.ui.textinput' )
local menu = {}

local t = 0
local wWidth = 800
local wHeight = 600
local canvas = lg.newCanvas()

local selectedButtonIdx = nil

local gradientQuad = lg.newMesh{
  {	0, 0,	0, 0,	0.4, 0.1, 0.4, 0.0 },
  {	1, 0, 1, 0,	0.8, 0.3, 0.1, 1.0 },
  {	1, 1,	1, 1,	0.7, 0.4, 0.1, 1.0 },
  {	0, 1,	0, 1,	0.4, 0.1, 0.3, 0.0 }, 
}

local buttons = 
{
  
  button{
    x = 15, w = 400, h = 50,
    y = 115,
    text = strings.mainmenu_button,
    color = { 0.5, 0.2, 0.8, 0.6 },
    callback = function() return scene.mainmenu() end },
  
  button{ 
    x = 15, w = 400, h = 50,
    y = 115 + 55,
    option = 'name',
    text = strings.option_name,
    color = { 0.5, 0.2, 0.2, 0.6 },
    callback = menu.textOption },

  button{ 
    x = 15, w = 400, h = 50,
    y = 115 + 55 * 2,
    option = 'pronoun',
    text = strings.option_pron,
    color = { 0.5, 0.2, 0.4, 0.6 },
    callback = menu.textOption },

  button{ 
    x = 15, w = 400, h = 50,
    y = 115 + 55 * 3,
    option = 'colour',
    text = strings.option_tint,
    color = { 0.5, 0.2, 0.6, 0.6 },
    callback = menu.colourOption },

  
  button{
    x = 15, w = 400, h = 50,
    y = 115 + 55 * 4,
    option = 'keybinds',
    text = strings.keybinding_button,
    color = { 0.8, 0.0, 0.4, 0.8 },
    callback = menu.editKeybinds,
    }
}

--[[local keybindButtons = {
  button{
    x = wWidth / 2, w = 400, h = 50
    y = 115,
    text = options:get( 'forward_key' ),
    color = { 0.8, 0.0, 0.4, 0.8 },
    callback = function() options:set( 'forward_key' ) end,
  },
  
  button{
    x = wWidth / 2, w = 400, h = 50
    y = 115,
    text = strings.backward_key,
    color = { 0.8, 0.0, 0.4, 0.8 },
    callback = function() return setOption( 'backward_key' ) end,
  }
  
}]]

function menu.onLoad()
  lg.setNewFont( 28 )
  return menu.resize( lg.getDimensions() )
end

function menu.draw()
  lg.setCanvas()
  lg.setColor( 1,1,1,1 )
  lg.draw( canvas )
end

function menu.paint()

  lg.setCanvas( canvas )

  --bg
  lg.setColor( 0.8, 0.6, 0.3, 1 )
  lg.rectangle( "fill", 0, 0, wWidth, wHeight )

  --buttons
  for id, button in pairs( buttons ) do button:draw( ) end

  --gradient
  lg.setColor( 1, 1, 1, 1 )
  lg.draw( gradientQuad, 0, 0, 0, wWidth, wHeight )
  lg.setCanvas()

end

function menu.update( dt )
  t = t + dt
end

function menu.resize( w, h )
  wWidth, wHeight = w, h
  canvas = lg.newCanvas()
  for id, uiButton in pairs( buttons ) do
    uiButton.w = wWidth
  end
  return menu.paint()
end

function menu.mousemoved( x, y, dx, dy, istouch )
  if selectedButtonIdx then
    buttons[selectedButtonIdx].selected = false
    selectedButtonIdx = nil
  end
  for id, menuButton in ipairs( buttons ) do
    if menuButton:contains( x, y ) then
      menuButton.selected = true
      selectedButtonIdx = id
      return menu.paint()
    end
  end
  
end

function menu.mousepressed( x, y, button, istouch, presses )
  if not selectedButtonIdx then return end
  local uiButton = buttons[selectedButtonIdx]
  if uiButton:contains( x, y ) then return uiButton(  ) end
  
  return menu.paint()
end

function menu.keypressed( key, code, isrepeat )

  if code == "escape" then
    return scene.mainmenu()
  end

  if code == "return" and selectedButtonIdx then
    return buttons[selectedButtonIdx].callback()
  end

  if code == "down" or code == "tab" or code == "up" then
    local sbi = (selectedButtonIdx or 1)
    buttons[sbi].selected = false

    --Increment / decrement
    sbi = sbi + ((code == "up") and -1 or 1)
    if sbi > #buttons then sbi = 1 end
    if sbi < 1 then sbi = #buttons end

    --Assign
    print( "selecting button: ", sbi )
    selectedButtonIdx = sbi
    buttons[selectedButtonIdx].selected = true
  end

  return menu.paint()
end

scene.options = menu
return menu